<?php
class OC_GROUP_PAM extends OC_Group_Backend {

    function __construct() {
        $pam_admin_groups = OC_Appconfig::getValue('user_pam', 'pam_admin_groups','');
        // Get list of users in the admin group
        $this->admin_groups = explode(",", $pam_admin_groups);
        $this->admin_groups = array_map('trim', $this->admin_groups);
        $this->admin_members = array();
        foreach( $this->admin_groups as $admin_group) {
            $groupinfo = posix_getgrnam($admin_group);
            if( $groupinfo) {
                $this->admin_members =  array_merge($this->admin_members, $groupinfo['members']);
            }
        }
        $this->admin_members = array_unique($this->admin_members);
    }

    public function inGroup($user, $group) {
        /*  If we are checking if a user is admin, we don't actually check against the admin group.
         *   Instead, we have a list of blessed groups. If the user is in any of them, they
         *   are considered an admin.
         */
        if ($group == 'admin') {
            // Check if this user has one of the trusted groups as his secondary group/
            // We already know all users that is true for, so see if this is one of
            //  those users.
            $members = $this->admin_members;
            if (in_array($user, $members)) {
                return true;
            // If the user doesn't have a trusted group as a secondary group, they might
            //  have it as a primary group. Check their gid, get the group name, see if its trusted.
            } else {
                $user_info = posix_getpwnam($user);
                if ($user_info) {
                    $primary_group = posix_getgrgid($user_info['gid']);
                    // If their primary group is one of the admin groups, they are good
                    if ($primary_group and in_array($primary_group['name'], $this->admin_groups)) {
                        return true;
                    }
                }

            }
            // If we get here, they weren't found to be good, so they are not an admin.
            return false;
        } else {
            $user_groups = getUserGroups($user);
            return in_array($group, $user_groups);
        }
    }

    /**
      Return groups user belongs to
      @param $uid Name of user
      @return array with group names
     */
    public function getUserGroups($uid) {
        require_once('ldap.inc.php');
        $dn = authenticate();
        if($dn){
            $groups = get_user_groups($dn, $uid);
            return $groups;
        } else {
            return array();
        }
    }


    /**
     * @brief get a list of all users in a group
     * @returns array with user ids
     */
    /*public function usersInGroup($gid) {
        //TODO: : return users in group
        return array();
    }*/


    /**
     * @brief get a list of all groups
     * @returns array with group names
     *
     * Returns a list with all groups
     */
	public function getGroups($search = '', $limit = -1, $offset = 0) {
        require_once('ldap.inc.php');
        $ds = authenticate();
        if($ds){
            $groups = get_grouplist($ds, $search);
            return $groups;
        } else {
            return array();
        }
    }


    /**
     * check if a group exists
     * @param string $gid
     * @return bool
     */
    public function groupExists($gid){
        return in_array($gid, $this->getGroups());
    }
}

?>
