<?php

/**
* Checks to see of a string contains a particular substring.
* We use this simple helper function to avoid the temptation to do if (strpos($needle, $haystack)) {}
* which will juggle types and return false when the needle is at location 0 in hackstack.
*/
function contains($substring, $string) {
        // Everything contains empty string.
        if ($substring === '') {
            return true;
        }
        $pos = strpos($string, $substring);
        if($pos === false) {
                return false;
        } else {
                return true;
        }
}

//function authenticate($username,$password)
function authenticate()
{
    static $ds;
    static $authed = false;
    // If we we have a good connection, use it
    if($authed) {
        return $ds;
    }
    $ds = ldap_connect(OC_APPconfig::getValue('user_pam','auth_ldap_server'),OC_Appconfig::getValue('user_pam','auth_ldap_port'));

    if (OC_Appconfig::getValue('user_pam','auth_ldap_starttls') == 'require') {
        $tls = ldap_start_tls($ds);
        if (OC_Appconfig::getValue('user_pam','auth_ldap_starttls') == 'require' && $tls == FALSE) {
            die("<h2>Fatal error: LDAP TLS required but not successfully negotiated:" . ldap_error($ds) . "</h2>");
        }
    }

    if ($ds)
    {
        if (OC_Appconfig::getValue('user_pam','auth_ldap_version'))
        {
            ldap_set_option($ds, LDAP_OPT_PROTOCOL_VERSION, OC_Appconfig::getValue('user_pam','auth_ldap_version'));
        }
        if (ldap_bind($ds))
        {
            if (!OC_Appconfig::getValue('user_pam','auth_ldap_group'))
            {
                $authed = true;
                return $ds;
            }
        }
        else
        {
            error_log(ldap_error($ds));
        }
    }
    else
    {
        error_log("Failed to connect to " . OC_APPconfig::getValue('user_pam','auth_ldap_server'). " on port " .OC_Appconfig::getValue('user_pam','auth_ldap_port'));

    }

    return 0;
}

function get_userlist($ds, $search_term = '')
{
    $filter = '(' . OC_Appconfig::getValue('user_pam','auth_ldap_prefix') . "*)";
    $search = ldap_search($ds, trim(OC_Appconfig::getValue('user_pam','auth_ldap_suffix'),','), $filter);
    $entries = ldap_get_entries($ds, $search);
    $userlist = array();
    if ($entries['count'])
    {
        foreach ($entries as $entry)
        {
            $username = $entry['uid'][0];
            $user_id  = $entry['uidnumber'][0];
            $primary_group_id = $entry['gidnumber'][0];
            // Check if this user contains the string we are searching for
            if (!contains($search_term, $username)) {
                continue;
            }
            if (OC_Appconfig::getValue('user_pam', 'pam_min_uid') and $user_id < OC_Appconfig::getValue('user_pam', 'pam_min_uid')) {
                continue;
            }else if (OC_Appconfig::getValue('user_pam', 'pam_max_uid') and $user_id > OC_Appconfig::getValue('user_pam', 'pam_max_uid')) {
                continue;
            }else if (OC_Appconfig::getValue('user_pam', 'pam_min_gid') and $primary_group_id < OC_Appconfig::getValue('user_pam', 'pam_min_gid')) {
                continue;
            }else if (OC_Appconfig::getValue('user_pam', 'pam_max_gid') and $primary_group_id > OC_Appconfig::getValue('user_pam', 'pam_max_gid')) {
                continue;
            }
            $userlist[] = $username;
        }
    }
    return $userlist;
}

function get_user_groups($ds, $username) {
    $prim_group_info = posix_getgrgid(get_primary_group($ds,$username));
    $primary_group = $prim_group_info['name'];
    $secondary_groups = get_secondary_group($ds,$username);
    array_push($secondary_groups, $primary_group);
    return $secondary_groups;
}

function get_primary_group($ds, $username) {
    $filter = '(' . OC_Appconfig::getValue('user_pam','auth_ldap_prefix') . "$username)";

    $search = ldap_search($ds, trim(OC_Appconfig::getValue('user_pam','auth_ldap_suffix'),','), $filter);
    $entries = ldap_get_entries($ds, $search);
    if ($entries['count'])
    {
        foreach ($entries as $entry) {
            if ($entry['gidnumber'][0]) {
                return $entry['gidnumber'][0];
            }
        }
    }
}

function get_secondary_group($ds, $username) {
    $filter = "(memberUid=$username)";

    $search = ldap_search($ds, trim(OC_Appconfig::getValue('user_pam','auth_ldap_groupbase'),','), $filter);
    $entries = ldap_get_entries($ds, $search);
    $groups = array();
    if ($entries['count'])
    {
        foreach ($entries as $entry)
        {
            if (isset($entry['cn'][0])) {
                array_push($groups, $entry['cn'][0]);
            }
        }
    }
    return $groups;
}

function get_grouplist($ds, $search_term = '') {
    $filter = "(cn=*)";
    $search = ldap_search($ds, trim(OC_Appconfig::getValue('user_pam','auth_ldap_groupbase'),','), $filter);
    $entries = ldap_get_entries($ds, $search);
    $groups = array();
    if ($entries['count'])
    {
        foreach ($entries as $entry)
        {
            if (isset($entry['cn'][0]) and contains($search_term, $entry['cn'][0])) {
                $primary_group_id = $entry['gidnumber'][0];
                if (OC_Appconfig::getValue('user_pam', 'pam_min_gid') and $primary_group_id < OC_Appconfig::getValue('user_pam', 'pam_min_gid')) {
                    continue;
                }else if (OC_Appconfig::getValue('user_pam', 'pam_max_gid') and $primary_group_id > OC_Appconfig::getValue('user_pam', 'pam_max_gid')) {
                    continue;
                }
                array_push($groups, $entry['cn'][0]);
            }
        }
    }
    return $groups;
}


?>
